import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:syncrow_web/pages/device_managment/all_devices/models/devices_model.dart';
import 'package:syncrow_web/pages/routines/bloc/functions_bloc/functions_bloc_bloc.dart';
import 'package:syncrow_web/pages/routines/bloc/routine_bloc/routine_bloc.dart';
import 'package:syncrow_web/pages/routines/models/device_functions.dart';
import 'package:syncrow_web/pages/routines/widgets/dialog_footer.dart';
import 'package:syncrow_web/pages/routines/widgets/dialog_header.dart';

class CeilingSensorDialog extends StatefulWidget {
  const CeilingSensorDialog({
    required this.uniqueCustomId,
    required this.functions,
    required this.deviceSelectedFunctions,
    required this.device,
    required this.dialogType,
    super.key,
  });
  final String? uniqueCustomId;
  final List<DeviceFunction> functions;
  final List<DeviceFunctionData> deviceSelectedFunctions;
  final AllDevicesModel? device;
  final String dialogType;

  @override
  State<CeilingSensorDialog> createState() => _CeilingSensorDialogState();
}

class _CeilingSensorDialogState extends State<CeilingSensorDialog> {
  @override
  Widget build(BuildContext context) {
    return AlertDialog(
      contentPadding: EdgeInsets.zero,
      content: BlocBuilder<FunctionBloc, FunctionBlocState>(
        builder: (context, state) {
          final selectedFunction = state.selectedFunction;
          final selectedOperationName = state.selectedOperationName;
          final selectedFunctionData = state.addedFunctions
              .firstWhere((f) => f.functionCode == selectedFunction,
                  orElse: () => DeviceFunctionData(
                        entityId: '',
                        functionCode: selectedFunction ?? '',
                        operationName: '',
                        value: null,
                      ));

          return Container(
            width: selectedFunction != null ? 600 : 360,
            height: 450,
            decoration: BoxDecoration(
              color: Colors.white,
              borderRadius: BorderRadius.circular(20),
            ),
            padding: const EdgeInsets.only(top: 20),
            child: Column(
              mainAxisSize: MainAxisSize.min,
              children: [
                const DialogHeader('Presence Sensor Condition'),
                DialogFooter(
                  onCancel: () => Navigator.pop(context),
                  onConfirm: state.addedFunctions.isNotEmpty
                      ? () {
                          context.read<RoutineBloc>().add(
                                AddFunctionToRoutine(
                                    state.addedFunctions, '{uniqueCustomId}'),
                              );

                          Navigator.pop(context, {
                            // 'deviceId': functions.first.deviceId,
                          });
                        }
                      : null,
                  isConfirmEnabled: selectedFunction != null,
                ),
              ],
            ),
          );
        },
      ),
    );
  }
}
